/***************************************************************************
 *   Copyright (C) 2008 by Daniel Calviño Sánchez                          *
 *   danxuliu@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef WAITFORSIGNAL_H
#define WAITFORSIGNAL_H

#include "ktutorial_export.h"

#include "WaitFor.h"

/**
 * Waits for a signal to be emitted.
 * When the signal is emitted and the WaitForSignal is active, the wait ends.
 *
 * Note that if the signal is emitted while the WaitFor isn't active, it won't
 * be registered and the condition won't be met. In order to met the condition,
 * the signal must be emitted while the WaitForSignal is active.
 */
class KTUTORIAL_EXPORT WaitForSignal: public WaitFor {
Q_OBJECT
public:

    /**
     * Creates a new WaitForSignal.
     * Note that the signal name must be the one "returned" by SIGNAL macro.
     *
     * @param sender The sender of the signal.
     * @param signal The signal to wait for.
     */
    //TODO maybe the SIGNAL part must be added here, instead of asuming that the
    //caller set it. The macro won't work, as SIGNAL(signal) would return the
    //string "signal" with the SIGNAL prefix. However, as SIGNAL macro adds a
    //number as a prefix and C++ methods can't begin with a number, it can be
    //infered whether the SIGNAL macro was used in the string passed or not, and
    //add it if needed.
    WaitForSignal(QObject* sender, const QString& signal): WaitFor(),
        mConditionMet(false) {
        connect(sender, signal.toStdString().c_str(), this, SLOT(signalWaitEnd()));
    }

    /**
     * Returns true if the connected signal was emitted while active, false
     * otherwise.
     *
     * @return True if the connected signal was emitted while active, false
     * otherwise.
     */
    virtual bool conditionMet() const {
        return mConditionMet;
    }

    /**
     * Sets this WaitForSignal active or inactive.
     * Activating it resets its condition.
     *
     * @param active True to set it active, false otherwise.
     */
    virtual void setActive(bool active) {
        WaitFor::setActive(active);

        if (active) {
            mConditionMet = false;
        }
    }

public slots:

    /**
     * Notifies that the wait for the signal ended.
     * If active, it sets the condition as met and ends its wait.
     */
    void signalWaitEnd();

private:

    /**
     * Whether the connected signal was received when active or not.
     */
    bool mConditionMet;

};

#endif
